using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;
using FarseerGames.FarseerPhysics.Factories;

namespace DestructionGolf
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class DestructionGolfGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        //The physics simulator is always needed. We supply a gravity of 250 units
        PhysicsSimulator simulator = new PhysicsSimulator(new Vector2(0, 250));

        // Game World

        /// <summary>
        /// Font for drawing messages etc
        /// </summary>
        SpriteFont font;

        /// <summary>
        /// Contains the scrolling background
        /// </summary>
        Texture2D backTexture;
        /// <summary>
        /// Contains the draw size of the background. 
        /// Will be wider than the screen so we can pan it.
        /// </summary>
        Rectangle backRectangle;
        /// <summary>
        /// Maximum scroll speed when the user is panning left and right
        /// </summary>
        float backScrollSpeed;

        /// <summary>
        /// Texture of the single target
        /// </summary>
        Texture2D targetTexture;
        Body targetBody;
        Geom targetGeom;
        Vector2 targetOrigin;

        /// <summary>
        /// Texture of the shelf the target sits on
        /// </summary>
        Texture2D shelfTexture;
        Body shelfBody;
        Geom shelfGeom;
        Vector2 shelfOrigin;

        /// <summary>
        /// Texture for the ball.
        /// </summary>
        Texture2D ballTexture;
        Body ballBody;
        Geom ballGeom;
        Vector2 ballOrigin;
        bool ballSoundActive = false;

        /// <summary>
        /// True when the ball is stopped and we
        /// are aiming the shot
        /// </summary>
        bool aimingShot;

        /// <summary>
        /// Used to transform the SpriteBatch draw of the 
        /// playfield. By putting a translation in this 
        /// matrix we can pan the whole screen left and right
        /// </summary>
        Matrix transformation;

        /// <summary>
        /// Amount of the translation being applied to the 
        /// transformation. When aiming this is controlled
        /// by the right thumstick. When the ball is flying
        /// this is controlled by the ball position so that
        /// the pan follows the ball.
        /// </summary>
        float xOffset = 0;

        // display dimensions
        /// <summary>
        /// Size of the screen. Used to scale the game objects
        /// and detect edges.
        /// </summary>
        int width;
        int height;

        /// <summary>
        /// Size of the margin between the ball and the edge of the
        /// screen when panning.
        /// </summary>
        int scrollMargin;


        // Sound Effects

        /// <summary>
        /// Played when the ball hits the target
        /// </summary>
        SoundEffect bangSound;

        public DestructionGolfGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Load the display font
            font = Content.Load<SpriteFont>("MessageFont");

            // Set up the sizes
            width = GraphicsDevice.Viewport.Width;
            height = GraphicsDevice.Viewport.Height;
            scrollMargin = width / 10;

            // Set the pan value to 0
            transformation = Matrix.CreateTranslation(0, 0, 0);

            backTexture = Content.Load<Texture2D>(@"images\Background");
            backRectangle = new Rectangle(0, 0, width * 5, height);
            backScrollSpeed = (float)width / 400;

            // Make a shelf to put our target on

            shelfTexture = Content.Load<Texture2D>(@"images\shelf");
            shelfOrigin = new Vector2(shelfTexture.Width / 2.0f, shelfTexture.Height / 2.0f);
            shelfBody = BodyFactory.Instance.CreateRectangleBody(shelfTexture.Width, shelfTexture.Height, 1);
            shelfBody.IsStatic = true;
            shelfBody.Position = new Vector2(550, 500);
            simulator.Add(shelfBody);

            shelfGeom = GeomFactory.Instance.CreateRectangleGeom(shelfBody, shelfTexture.Width, shelfTexture.Height);
            shelfGeom.RestitutionCoefficient = 0.3f;
            shelfGeom.RestitutionCoefficient = 0.3f;
            shelfGeom.FrictionCoefficient = 0.5f;
            simulator.Add(shelfGeom);

            targetTexture = Content.Load<Texture2D>(@"images\Target");
            targetOrigin = new Vector2(targetTexture.Width / 2.0f, targetTexture.Height / 2.0f);
            targetBody = BodyFactory.Instance.CreateRectangleBody(targetTexture.Width, targetTexture.Height, 3);
            targetBody.Position = new Vector2(550, 400);
            simulator.Add(targetBody);

            targetGeom = GeomFactory.Instance.CreateRectangleGeom(targetBody, targetTexture.Width, targetTexture.Height);
            targetGeom.RestitutionCoefficient = 0.3f;
            targetGeom.FrictionCoefficient = 0.5f;
            simulator.Add(targetGeom);

            ballTexture = Content.Load<Texture2D>(@"images\Ball");
            ballOrigin = new Vector2(ballTexture.Width / 2.0f, ballTexture.Height / 2.0f);
            ballBody = BodyFactory.Instance.CreateRectangleBody(ballTexture.Width, ballTexture.Height, 1f);
            ballBody.Position = new Vector2(40, 500);
            ballBody.IsStatic = true;
            simulator.Add(ballBody);
            resetBall();

            ballGeom = GeomFactory.Instance.CreateCircleGeom(ballBody, ballTexture.Width/2, 50);
            ballGeom.RestitutionCoefficient = 0.3f;
            ballGeom.RestitutionCoefficient = 0.3f;
            ballGeom.FrictionCoefficient = 0.5f;
            ballGeom.OnCollision += new CollisionEventHandler(ballHits);
            simulator.Add(ballGeom);

            aimingShot = true;

            bangSound = Content.Load<SoundEffect>(@"sounds\explodeSound");

            resetBall();
        }

        bool ballHits(Geom geometry1, Geom geometry2, ContactList contactList)
        {
            if (ballSoundActive)
            {
                return true;
            }
            if (geometry1 == shelfGeom || geometry2 == shelfGeom)
            {
                return true;
            }
            ballSoundActive = true;
            bangSound.Play();
            return true;
        }


        /// <summary>
        /// Puts the ball back in the starting position
        /// </summary>
        private void resetBall()
        {
            xOffset = 0;
            targetBody.Position = new Vector2(550, 400);
            targetBody.Rotation = 0;
            targetBody.LinearVelocity = Vector2.Zero;

            ballBody.Position = new Vector2(40, 500);
            ballBody.Rotation = 0;
            ballBody.LinearVelocity = Vector2.Zero;
            ballBody.IsStatic = true;
            aimingShot = true;
            ballSoundActive = false;
        }


        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            GamePadState gamePad = GamePad.GetState(PlayerIndex.One);

            if (aimingShot)
            {
                // Aiming the shot
                // Pan the display left and right using the right thumbstick
                // This changes the xOffset value that is used to translate
                // the screen
                xOffset = xOffset - gamePad.ThumbSticks.Right.X * backScrollSpeed;

                // Press A to fire the ball
                if (gamePad.Buttons.A == ButtonState.Pressed)
                {
                    aimingShot = false;
                    ballBody.IsStatic = false;
                    ballBody.ApplyImpulse(new Vector2(gamePad.ThumbSticks.Left.X * 500, -gamePad.ThumbSticks.Left.Y * 500));
                }
            }
            else
            {
                // Shot is in the air

                // Update the X and Y position of the ball based on its
                // present speed

                // Check if the ball has reached the right hand edge
                if (ballGeom.Position.X + scrollMargin > width)
                {
                    // need to scroll the screen
                    xOffset = width - (ballGeom.Position.X + scrollMargin);
                }


                // If the ball has gone off the bottom of the screen
                // reset it
                if (ballGeom.Position.Y > height || gamePad.Buttons.B == ButtonState.Pressed)
                {
                    resetBall();
                }

            }

            // Make a translation of the screen to perform the pan
            transformation = Matrix.CreateTranslation(xOffset, 0, 0);

            simulator.Update(gameTime.ElapsedGameTime.Milliseconds * .0005f);

            base.Update(gameTime);
        }

        private void drawString(string message)
        {
            int xPos = (int) ((width - font.MeasureString(message).X) / 2);
            spriteBatch.DrawString(font, message, new Vector2(xPos, 0), Color.White);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // Begin drawing with a transformed SpriteBatch

            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.None, transformation);

            spriteBatch.Draw(backTexture, backRectangle, Color.White);
            spriteBatch.Draw(targetTexture, targetGeom.Position, null, Color.White, targetGeom.Rotation, targetOrigin, 1, SpriteEffects.None, 1);
            spriteBatch.Draw(shelfTexture, shelfGeom.Position, null, Color.White, shelfGeom.Rotation, shelfOrigin, 1, SpriteEffects.None, 1);
            spriteBatch.Draw(ballTexture, ballGeom.Position, null, Color.White, ballGeom.Rotation, ballOrigin, 1, SpriteEffects.None, 1);

            spriteBatch.End();

            // This draw will not be scrolled, so all this content
            // will always be displayed

            spriteBatch.Begin();

            drawString("Destruction Golf");

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
